@@ -67,7 +67,7 @@ use handler::{RequestProtocol, RequestResponseHandler, RequestResponseHandlerEve
 use libp2p_core::{connection::ConnectionId, ConnectedPoint, Multiaddr, PeerId};
 use libp2p_swarm::{
     dial_opts::{self, DialOpts},
-    DialError, IntoProtocolsHandler, NetworkBehaviour, NetworkBehaviourAction, NotifyHandler,
+    DialError, IntoConnectionHandler, NetworkBehaviour, NetworkBehaviourAction, NotifyHandler,
     PollParameters,
 };
 use smallvec::SmallVec;
@@ -567,10 +567,10 @@ impl<TCodec> NetworkBehaviour for RequestResponse<TCodec>
 where
     TCodec: RequestResponseCodec + Send + Clone + 'static,
 {
-    type ProtocolsHandler = RequestResponseHandler<TCodec>;
+    type ConnectionHandler = RequestResponseHandler<TCodec>;
     type OutEvent = RequestResponseEvent<TCodec::Request, TCodec::Response>;
 
-    fn new_handler(&mut self) -> Self::ProtocolsHandler {
+    fn new_handler(&mut self) -> Self::ConnectionHandler {
         RequestResponseHandler::new(
             self.inbound_protocols.clone(),
             self.codec.clone(),
@@ -614,21 +614,13 @@ where
         connection.address = new_address;
     }
 
-    fn inject_connected(&mut self, peer: &PeerId) {
-        if let Some(pending) = self.pending_outbound_requests.remove(peer) {
-            for request in pending {
-                let request = self.try_send_request(peer, request);
-                assert!(request.is_none());
-            }
-        }
-    }
-
     fn inject_connection_established(
         &mut self,
         peer: &PeerId,
         conn: &ConnectionId,
         endpoint: &ConnectedPoint,
         _errors: Option<&Vec<Multiaddr>>,
+        other_established: usize,
     ) {
         let address = match endpoint {
             ConnectedPoint::Dialer { address, .. } => Some(address.clone()),
@@ -638,6 +630,15 @@ where
             .entry(*peer)
             .or_default()
             .push(Connection::new(*conn, address));
+
+        if other_established == 0 {
+            if let Some(pending) = self.pending_outbound_requests.remove(peer) {
+                for request in pending {
+                    let request = self.try_send_request(peer, request);
+                    assert!(request.is_none());
+                }
+            }
+        }
     }
 
     fn inject_connection_closed(
@@ -645,7 +646,8 @@ where
         peer_id: &PeerId,
         conn: &ConnectionId,
         _: &ConnectedPoint,
-        _: <Self::ProtocolsHandler as IntoProtocolsHandler>::Handler,
+        _: <Self::ConnectionHandler as IntoConnectionHandler>::Handler,
+        remaining_established: usize,
     ) {
         let connections = self
             .connected
@@ -658,6 +660,7 @@ where
             .map(|p: usize| connections.remove(p))
             .expect("Expected connection to be established before closing.");
 
+        debug_assert_eq!(connections.is_empty(), remaining_established == 0);
         if connections.is_empty() {
             self.connected.remove(peer_id);
         }
@@ -685,14 +688,10 @@ where
         }
     }
 
-    fn inject_disconnected(&mut self, peer: &PeerId) {
-        self.connected.remove(peer);
-    }
-
     fn inject_dial_failure(
         &mut self,
         peer: Option<PeerId>,
-        _: Self::ProtocolsHandler,
+        _: Self::ConnectionHandler,
         _: &DialError,
     ) {
         if let Some(peer) = peer {
@@ -873,7 +872,7 @@ where
         &mut self,
         _: &mut Context<'_>,
         _: &mut impl PollParameters,
-    ) -> Poll<NetworkBehaviourAction<Self::OutEvent, Self::ProtocolsHandler>> {
+    ) -> Poll<NetworkBehaviourAction<Self::OutEvent, Self::ConnectionHandler>> {
         if let Some(ev) = self.pending_events.pop_front() {
             return Poll::Ready(ev);
         } else if self.pending_events.capacity() > EMPTY_QUEUE_SHRINK_THRESHOLD {
